@@ -12,7 +12,7 @@ from inspect import currentframe
 from itertools import chain, product
 from six import binary_type, string_types, text_type, BytesIO
 from string import *
-from types import FunctionType
+from types import FunctionType, ModuleType
 try: # Python3
     from importlib import reload
 except ImportError:
@@ -29,7 +29,8 @@ except ImportError:  # Python 3
 
 __all__ = ["add", "add_map", "b", "clear", "codecs", "decode", "encode", "ensure_str", "examples", "guess",
            "generate_strings_from_regex", "get_alphabet_from_mask", "handle_error", "list_categories", "list_encodings",
-           "lookup", "maketrans", "re", "register", "remove", "reset", "s2i", "search", "BytesIO", "MASKS", "PY3"]
+           "lookup", "maketrans", "re", "register", "remove", "reset", "s2i", "search", "stopfunc", "BytesIO", "MASKS",
+           "PY3"]
 CODECS_REGISTRY = None
 MASKS = {
     'a': printable,
@@ -46,8 +47,7 @@ PY3 = sys.version[0] == "3"
 __codecs_registry = []
 
 
-entropy      = lambda s: -sum([p * log(p, 2) for p in [float(s.count(c)) / len(s) for c in set(s)]])
-is_printable = lambda s: all(c in printable for c in ensure_str(s))
+entropy = lambda s: -sum([p * log(p, 2) for p in [float(s.count(c)) / len(s) for c in set(s)]])
 
 isb = lambda s: isinstance(s, binary_type)
 iss = lambda s: isinstance(s, string_types)
@@ -57,9 +57,8 @@ s2i = lambda s: int(codecs.encode(s, "base16"), 16)
 
 
 def add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=False, **kwargs):
-    """
-    This adds a new codec to the codecs module setting its encode and/or decode functions, eventually dynamically naming
-     the encoding with a pattern and with file handling.
+    """ This adds a new codec to the codecs module setting its encode and/or decode functions, eventually dynamically
+         naming the encoding with a pattern and with file handling.
     
     :param ename:         encoding name
     :param encode:        encoding function or None
@@ -165,10 +164,9 @@ def add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=
 
 
 def add_map(ename, encmap, repl_char="?", sep="", ignore_case=None, no_error=False, intype=None, outype=None, **kwargs):
-    """
-    This adds a new mapping codec (that is, declarable with a simple character mapping dictionary) to the codecs module
-     dynamically setting its encode and/or decode functions, eventually dynamically naming the encoding with a pattern
-     and with file handling (if text is True).
+    """ This adds a new mapping codec (that is, declarable with a simple character mapping dictionary) to the codecs
+         module dynamically setting its encode and/or decode functions, eventually dynamically naming the encoding with
+         a pattern and with file handling (if text is True).
     
     :param ename:         encoding name
     :param encmap:        characters encoding map ; can be a dictionary of encoding maps (for use with the first capture
@@ -197,11 +195,10 @@ def add_map(ename, encmap, repl_char="?", sep="", ignore_case=None, no_error=Fal
     
     def __generic_code(decode=False):
         def _wrapper(param):
-            """
-            The parameter for wrapping comes from the encoding regex pattern ; e.g.
-                [no pattern]           => param will be None everytime
-                r"barbie[-_]?([1-4])$" => param could be int 1, 2, 3 or 4
-                r"^morse(|[-_]?.{3})$" => param could be None, "-ABC" (for mapping to ".-/")
+            """ The parameter for wrapping comes from the encoding regex pattern ; e.g.
+                 [no pattern]           => param will be None everytime
+                 r"barbie[-_]?([1-4])$" => param could be int 1, 2, 3 or 4
+                 r"^morse(|[-_]?.{3})$" => param could be None, "-ABC" (for mapping to ".-/")
             
             In order of precedence:
             1. when param is a key in mapdict or mapdict is a list of encoding maps (hence in the case of "barbie...",
@@ -544,7 +541,7 @@ def ensure_str(s, encoding='utf-8', errors='strict'):
 # make conversion functions compatible with input/output strings/bytes
 def fix_inout_formats(f):
     """ This decorator ensures that the first output of f will have the same text format as the first input (str or
-     bytes). """
+         bytes). """
     @wraps(f)
     def _wrapper(*args, **kwargs):
         a0 = args[0]
@@ -562,7 +559,7 @@ def fix_inout_formats(f):
 # alphabet generation function from a given mask
 def get_alphabet_from_mask(mask):
     """ This function generates an alphabet from the given mask. The style used is similar to Hashcat ; group keys are
-     marked with a heading "?". """
+         marked with a heading "?". """
     i, alphabet = 0, ""
     while i < len(mask):
         c = mask[i]
@@ -579,8 +576,7 @@ def get_alphabet_from_mask(mask):
 
 # generic error handling function
 def handle_error(ename, errors, sep="", repl_char="?", repl_minlen=1, decode=False, item="position"):
-    """
-    This shortcut function allows to handle error modes given some tuning parameters.
+    """ This shortcut function allows to handle error modes given some tuning parameters.
     
     :param ename:       encoding name
     :param errors:      error handling mode
@@ -597,8 +593,7 @@ def handle_error(ename, errors, sep="", repl_char="?", repl_minlen=1, decode=Fal
     exec("class %s(ValueError): pass" % exc, glob)
     
     def _handle_error(token, position):
-        """
-        This handles an encoding/decoding error according to the selected handling mode.
+        """ This handles an encoding/decoding error according to the selected handling mode.
         
         :param token:    input token to be encoded/decoded
         :param position: token position index
@@ -661,9 +656,8 @@ codecs.lookup = lookup
 
 
 def register(search_function, add_to_codecs=False):
-    """
-    Register function for registering new codecs in the local registry of this module and, if required, in the native
-     codecs registry (for use with the built-in 'open' function).
+    """ Register function for registering new codecs in the local registry of this module and, if required, in the
+         native codecs registry (for use with the built-in 'open' function).
     
     :param search_function: search function for the codecs registry
     :param add_to_codecs:   also add the search function to the native registry
@@ -833,15 +827,72 @@ def generate_strings_from_regex(regex, star_plus_max=STAR_PLUS_MAX, repeat_max=R
 
 
 # guess feature objects
-def __guess(input, stop_func, depth, max_depth, codecs, result, found=()):
+stopfunc = ModuleType("stopfunc", """
+    Predefined stop functions
+    ~~~~~~~~~~~~~~~~~~~~~~~~~
+
+    This submodule contains stop functions for the guess feature of codext.
+    
+    - `flag`:       searches for the pattern "[Ff][Ll1][Aa4@][Gg9]" (either UTF-8 or UTF-16)
+    - `lang_**`:    checks if the given lang (any from the PROFILES_DIRECTORY of the langdetect module) is detected
+    - `printables`: checks that every output character is in the set of printables
+""")
+stopfunc.printables = lambda s: all(c in printable for c in ensure_str(s))
+
+try:
+    from langdetect import detect, PROFILES_DIRECTORY
+    for lang in [p.replace("-", "") for p in os.listdir(PROFILES_DIRECTORY)]:
+        setattr(stopfunc, "lang_%s" % lang, lambda s, l=lang: stopfunc.printables(s) and detect(s) == l)
+except ImportError:
+    pass
+
+
+__flag = lambda x: re.search(r"[Ff][Ll1][Aa4@][Gg9]", x) is not None
+def _flag(x):
+    try:
+        return __flag(ensure_str(b(x).decode("utf16")))
+    except (UnicodeDecodeError, UnicodeEncodeError):
+        return __flag(x)
+stopfunc.flag = _flag
+
+
+def __guess(input, stop_func, depth, max_depth, codec_categories, exclude, result, found=(), stop=True, show=False):
     """ Perform a breadth-first tree search using a ranking logic to select and prune the list of codecs. """
     if depth > 0 and stop_func(input):
+        if not stop and show:
+            s = "[*] %s: %s" % (", ".join(found), ensure_str(input))
+            print(s if len(s) <= 80 else s[:77] + "...")
         result.append((input, found))
         return
     if depth >= max_depth or len(result) > 0:
         return
-    for new_input, encoding in __rank(input, codecs):
-        __guess(new_input, stop_func, depth+1, max_depth, codecs, result, found + (encoding, ))
+    # compute included and excluded codecs for this depth
+    def __expand(items, descr=None, transform=None):
+        # format 1: when string, take it as the only items at any depth
+        if isinstance(items, string_types):
+            r = (items, )
+        # format 2: when tuple, consider it as a list of items at any depth
+        elif isinstance(items, tuple):
+            r = items
+        # format 3: when list, consider it as the list of tuples of items with the order number corresponding to the
+        #            applicable depth
+        elif isinstance(items, list):
+            try:
+                r = items[depth] or ()
+                if isinstance(r, string_types):
+                    r = (r, )
+            except IndexError:
+                r = ()
+        else:
+            raise ValueError("Bad %sformat %s" % (["%s " % descr, ""][descr is None], items))
+        return r if transform is None else transform(*r)
+    # parse valid encodings, expanding included/excluded codecs
+    c, e = __expand(codec_categories, "codec_categories", list_encodings), __expand(exclude, "exclude")
+    for new_input, encoding in __rank(input, c):
+        if encoding in e:
+            continue
+        __guess(new_input, stop_func, depth+1, max_depth, codec_categories, exclude, result, found + (encoding, ), stop,
+                show)
 
 
 def __rank(input, codecs):
@@ -874,7 +925,8 @@ def __score(input, codec):
         yield score, new_input, encoding
 
 
-def guess(input, stop_func=is_printable, max_depth=5, codec_categories=None, found=()):
+def guess(input, stop_func=stopfunc.printables, max_depth=5, codec_categories=None, exclude=None, found=(), stop=True,
+          show=False):
     """ Try decoding without the knowledge of the encoding(s). """
     if max_depth <= 0:
         raise ValueError("Depth must be a non-null positive integer")
@@ -884,18 +936,12 @@ def guess(input, stop_func=is_printable, max_depth=5, codec_categories=None, fou
     if isinstance(stop_func, string_types):
         p = stop_func
         stop_func = lambda s: re.search(ensure_str(p).lower(), ensure_str(s).lower()) is not None
-    if codec_categories is None:
-        codecs = list_encodings()
-    elif isinstance(codec_categories, string_types):
-        codecs = list_encodings(codec_categories)
-    elif isinstance(codec_categories, (tuple, list, set)):
-        codecs = list_encodings(*codec_categories)
     if len(input) > 0:
         result = []
         for d in range(max_depth):
-            __guess(input, stop_func, 0, d+1, codecs, result, tuple(found))
-            if len(result) > 0:
-                return result[0]
-    return None, None
+            __guess(input, stop_func, 0, d+1, codec_categories or [], exclude or [], result, tuple(found), stop, show)
+            if stop and len(result) > 0:
+                return result
+    return result
 codecs.guess = guess
 
